"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const inquirer = require("inquirer");
class BaseCommand extends command_1.default {
    constructor() {
        super(...arguments);
        this.interactive = false;
        this.jiraHost = "";
        this.jiraUser = "";
        this.jiraAccessToken = "";
    }
    async run() {
        const { flags } = this.parse(BaseCommand);
        this.interactive = flags["interactive"];
        this.jiraUser = await this.getFlagValue(flags, "jira-email");
        this.jiraHost = await this.getFlagValue(flags, "jira-host", "jira.atlassian.com");
        this.jiraAccessToken = await this.getFlagValue(flags, "jira-access-token");
    }
    listPrompter(choicesProvider) {
        return async (name) => {
            const choices = await choicesProvider.call(this);
            const answer = await inquirer.prompt([{ name: name, message: name, type: "list", choices: choices }]);
            return answer[name];
        };
    }
    valuePrompter(defaultValue) {
        return async (name) => {
            const answer = await inquirer.prompt([{ name: name, message: name, default: defaultValue }]);
            return answer[name];
        };
    }
    confirmPrompter(defaultValue) {
        return async (name) => {
            const answer = await inquirer.prompt([{ name: name, message: name, type: "confirm", default: defaultValue }]);
            return answer[name];
        };
    }
    async getFlagValue2(flags, flag, prompter, defaultValue) {
        const value = flags[flag];
        if (value) {
            return value;
        }
        else if (this.interactive) {
            return await prompter.call(this, flag);
        }
        else {
            return defaultValue || this.error(`Missing --${flag} flag!`);
        }
    }
    async getFlagValue(flags, flag, defaultValue) {
        const value = flags[flag];
        if (value) {
            return value;
        }
        else if (this.interactive) {
            const answer = await inquirer.prompt([{ name: flag, message: flag, default: defaultValue }]);
            return answer[flag];
        }
        else {
            return defaultValue || this.error(`Missing --${flag} flag!`);
        }
    }
}
BaseCommand.flags = {
    "jira-host": command_1.flags.string({ description: "custom host for jira" }),
    "jira-email": command_1.flags.string({ description: "email address associated with jira" }),
    "jira-access-token": command_1.flags.string({ description: "jira access token" }),
    "github-access-token": command_1.flags.string({ description: "github access token" }),
    "interactive": command_1.flags.boolean({ char: "i", description: "interactive mode", default: false })
};
exports.default = BaseCommand;
