"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const nodegit_1 = require("nodegit");
const jiraClient_1 = require("../jiraClient");
const githubClient_1 = require("../githubClient");
const base_1 = require("../base");
class Pr extends base_1.default {
    async run() {
        await super.run();
        const { flags } = this.parse(Pr);
        // JIRA
        const jiraTicketId = await this.getFlagValue2(flags, "ticket-id", this.valuePrompter());
        const jiraTicket = await this.getJiraTicket(this.jiraAccessToken, this.jiraHost, this.jiraUser, jiraTicketId);
        // GitHub
        const baseBranch = await this.getFlagValue2(flags, "base-branch", this.listPrompter(this.branchChoiceProvider), "master");
        const githubAccessToken = await this.getFlagValue2(flags, "github-access-token", this.valuePrompter());
        const prTitle = await this.getFlagValue2(flags, "pr-title", this.valuePrompter(jiraTicket.fields.summary), jiraTicket.fields.summary);
        const draft = await this.getFlagValue2(flags, "draft", this.confirmPrompter(false), false);
        const prTitleWithTicketId = this.createPRTitle(prTitle, jiraTicket);
        const prDescription = this.createPRDescription(this.jiraHost, jiraTicket);
        const pr = await this.makePullRequest(githubAccessToken, baseBranch, prTitleWithTicketId, prDescription, draft);
        this.log(pr.html_url);
    }
    async branchChoiceProvider() {
        const repo = await nodegit_1.Repository.open(".");
        return (await repo.getReferences(3 /* LISTALL */))
            .filter((ref) => { return ref.isBranch(); })
            .map((ref) => { return { name: ref.shorthand() }; });
    }
    async getJiraTicket(accessToken, host, username, ticketId) {
        let jiraClient = new jiraClient_1.JiraClient({
            username: username,
            accessToken: accessToken,
            host: host
        });
        cli_ux_1.default.action.start("Fetching JIRA ticket");
        const jiraTicket = await jiraClient.getJiraTicket(ticketId);
        cli_ux_1.default.action.stop("done");
        return jiraTicket;
    }
    async makePullRequest(accessToken, base, title, description, draft) {
        let githubClient = new githubClient_1.GitHubClient(accessToken);
        cli_ux_1.default.action.start("Making pull request");
        const result = await githubClient.openPullRequest({
            repo: await nodegit_1.Repository.open("."),
            title: title,
            description: description,
            base: base,
            draft: draft
        });
        cli_ux_1.default.action.stop("done");
        return result;
    }
    createPRTitle(title, jiraTicket) {
        return `[${jiraTicket.key}] ${title}`;
    }
    createPRDescription(jiraHost, jiraTicket) {
        return `https://${jiraHost}/browse/${jiraTicket.key}`;
    }
}
Pr.description = "Create GitHub PRs from JIRA tickets";
Pr.flags = Object.assign({}, base_1.default.flags, { "help": command_1.flags.help({ char: "h" }), "base-branch": command_1.flags.string({ char: "b", description: "base branch for PR" }), "ticket-id": command_1.flags.string({ char: "t", description: "jira ticket ID" }), "pr-title": command_1.flags.string({ description: "custom PR title" }), "draft": command_1.flags.boolean({ description: "draft PR" }) });
exports.default = Pr;
